local AceGUI = LibStub("AceGUI-3.0")

local rankPick, playerPick, charPick, addCharName, changeMain, removeChar;

local alts = {};
local players = {};
local ranks = {};
local newAlt = nil;
local oldMain, newMain;

function MorthalinRaidTools:ConstructAltsSettings()
	self.ui.altsSettings = AceGUI:Create("Frame");
	self.ui.altsSettings:SetTitle("Buffs settings");
	self.ui.altsSettings:SetStatusText("");
	self.ui.altsSettings:SetLayout("Flow");
	self.ui.altsSettings:SetWidth(270);
	self.ui.altsSettings:SetHeight(240);
	self.ui.altsSettings:EnableResize(false);
	self.ui.altsSettings.frame:EnableMouse(true);
	self.ui.altsSettings.frame:SetUserPlaced(true);
	
	rankPick = AceGUI:Create("Dropdown");
	rankPick:SetLabel("Showed ranks");
	rankPick:SetWidth(220);
	rankPick:SetMultiselect(true);
	rankPick:SetCallback("OnValueChanged", function(obj, func, index, value) ranks[rankPick.list[index]] = value; MorthalinRaidTools:LoadPlayers(); end);
	self.ui.altsSettings:AddChild(rankPick);
	
	playerPick = AceGUI:Create("Dropdown");
	playerPick:SetLabel("Players");
	playerPick:SetWidth(220);
	playerPick:SetCallback("OnValueChanged", function(obj, func, value) MorthalinRaidTools:LoadCharacters(); end);
	self.ui.altsSettings:AddChild(playerPick);
	
	charPick = AceGUI:Create("Dropdown");
	charPick:SetLabel("Characters");
	charPick:SetWidth(180);
	charPick:SetCallback("OnValueChanged", function(self, func, key, checked)  end);
	self.ui.altsSettings:AddChild(charPick);
	
	changeMain = AceGUI:Create("Icon");
	changeMain:SetImage("Interface\\Icons\\spell_holy_divinespirit");
	changeMain:SetWidth(20);
	changeMain:SetHeight(20);
	changeMain:SetImageSize(15,15);
	changeMain:SetCallback("OnClick", function() MorthalinRaidTools:ChangeMain(); end);
	changeMain:SetCallback("OnEnter", function()
		GameTooltip:SetOwner(changeMain.frame,"ANCHOR_BOTTOMRIGHT");
		GameTooltip:AddLine("Make selected character as Main");
		GameTooltip:Show();
	end);
	changeMain:SetCallback("OnLeave", function() self:HideTooltip(); end);
	self.ui.altsSettings:AddChild(changeMain);
	
	removeChar = AceGUI:Create("Icon");
	removeChar:SetImage("Interface\\Icons\\spell_chargenegative");
	removeChar:SetWidth(20);
	removeChar:SetHeight(20);
	removeChar:SetImageSize(15,15);
	removeChar:SetCallback("OnClick", function() MorthalinRaidTools:RemoveAlt(); end);
	removeChar:SetCallback("OnEnter", function()
		GameTooltip:SetOwner(changeMain.frame,"ANCHOR_BOTTOMRIGHT");
		GameTooltip:AddLine("Remove selected character as Alt");
		GameTooltip:Show();
	end);
	removeChar:SetCallback("OnLeave", function() self:HideTooltip(); end);
	self.ui.altsSettings:AddChild(removeChar);
	
	addCharName = AceGUI:Create("EditBox");
	addCharName:SetLabel("Add character");
	addCharName:SetWidth(220);
	addCharName:DisableButton(true);
	addCharName:SetCallback("OnTextChanged", function(widget, event, text)
		if(string.len(text) ~= 0) then
			self:ConstructWhisperer(addCharName, text, 3);
		else
			self:HideWhisperer();
		end
	end)
	addCharName:SetCallback("OnEnterPressed", function(widget, event, text) self:AddAlt(text); addCharName:SetText(""); end)
	self.ui.altsSettings:AddChild(addCharName);
	
	self.ui.altsSettings:Hide();
end

function MorthalinRaidTools:ShowAltSettings()
	if(self.ui.altsSettings:IsVisible() == false) then
		self.ui.altsSettings:Show();
		MorthalinRaidTools:LoadRanks();
		MorthalinRaidTools:LoadPlayers();
	end
end

function MorthalinRaidTools:HideAltSettings()
	if(self.ui.altsSettings:IsVisible()) then
		self.ui.altsSettings:Hide();
	end
end

function MorthalinRaidTools:LoadPlayers()
	local index = 1;
	playerPick:SetValue(nil);
	playerPick:SetList(nil);
	charPick:SetValue(nil);
	charPick:SetList(nil);
	addCharName:SetDisabled(true);
	changeMain:SetDisabled(true);
	removeChar:SetDisabled(true);
	alts = {};
	players = {};
	for i=1, GetNumGuildMembers() do
		local name, rank, rankIndex, level, class, zone, note, officernote, online, status = GetGuildRosterInfo(i);
		
		if(string.sub(officernote, 1, 4) == "Alt:") then
			if(players[string.sub(officernote, 5, -1)] == nil) then
				players[string.sub(officernote, 5, -1)] = {name = string.sub(officernote, 5, -1), class = "Unknown", rank = "Unknown", index = 0, alts = {}};
			end
			
			if(alts[string.sub(name, 1, name:find("-") - 1)] == nil) then
				alts[string.sub(name, 1, name:find("-") - 1)] = string.sub(officernote, 5, -1);
				table.insert(players[string.sub(officernote, 5, -1)].alts, {name = string.sub(name, 1, name:find("-") - 1), class = class, index = i});
			end
		else
			if(players[string.sub(name, 1, name:find("-") - 1)] == nil) then
				players[string.sub(name, 1, name:find("-") - 1)] = {name = string.sub(name, 1, name:find("-") - 1), class = class, rank = rank, index = i, alts = {}};
			else
				players[string.sub(name, 1, name:find("-") - 1)].class = class;
				players[string.sub(name, 1, name:find("-") - 1)].rank = rank;
				players[string.sub(name, 1, name:find("-") - 1)].index = i;
			end
		end
	end
	
	playersIndexed = self:ConvertStringIndex(players);
	local function sortPlayers(a, b)
		return a.name < b.name;
	end
	table.sort(playersIndexed, sortPlayers);
	for i=1, #playersIndexed do
		if(ranks[playersIndexed[i].rank]) then
			playerPick:AddItem(index, "|cff" .. MorthalinRaidTools.classColors[playersIndexed[i].class].hex .. playersIndexed[i].name);
			index = index + 1;
		end
	end
end

function MorthalinRaidTools:LoadRanks()
	local index = 1;
	ranks = {};
	rankPick:SetValue(nil);
	rankPick:SetList(nil);
	for i = 1, GuildControlGetNumRanks() do
		rankPick:AddItem(i, GuildControlGetRankName(i));
		ranks[GuildControlGetRankName(i)] = true;
		index = i + 1;
	end
	
	rankPick:AddItem(index, "Unknown");
	ranks["Unknown"] = true;
	
	local text;
	for i, widget in rankPick.pullout:IterateItems() do
		if widget.type == "Dropdown-Item-Toggle" then
			widget.value = true;
			widget.check:Show();
			if text then
				text = text..", "..widget:GetText();
			else
				text = widget:GetText();
			end
		end
	end
	rankPick:SetText(text);
end

function MorthalinRaidTools:LoadCharacters()
	local index = 1;
	charPick:SetValue(nil);
	charPick:SetList(nil);
	charPick:AddItem(index, playerPick.list[playerPick:GetValue()]);
	index = index + 1;
	charPick:SetValue(1);
	if(playerPick.list[playerPick:GetValue()] ~= nil and players[playerPick.list[playerPick:GetValue()]:sub(11,-1)] ~= nil) then
		for i = 1, #players[playerPick.list[playerPick:GetValue()]:sub(11,-1)].alts do
			charPick:AddItem(index, "|cff" .. MorthalinRaidTools.classColors[players[playerPick.list[playerPick:GetValue()]:sub(11,-1)].alts[i].class].hex .. players[playerPick.list[playerPick:GetValue()]:sub(11,-1)].alts[i].name);
			index = index + 1;
		end
	end
	addCharName:SetDisabled(false);
	changeMain:SetDisabled(false);
	removeChar:SetDisabled(false);
end

function MorthalinRaidTools:HaveAlts(name)	
	if(players[name] == nil or #players[name].alts == 0) then
		return false;
	else
		return true;
	end
end

function MorthalinRaidTools:IsAlt(name)
	if(alts[name] ~= nil) then
		return true;
	else
		return false;
	end
end

function MorthalinRaidTools:AddAlt(name)
	self:HideWhisperer();
	addCharName:SetText("");
	newAlt = name;
	if(MorthalinRaidTools:HaveAlts(name)) then
		MorthalinRaidTools:Print("|cffff0000Player have alts and cannot be added as alt!");
	else
		if(MorthalinRaidTools:IsAlt(name)) then
			self:ConstructConfirm("Alts", "Character is already an alt. Do you want to change his main?", "Alts", 9);
		else
			MorthalinRaidTools:AddAltContinue();
		end
	end
end

function MorthalinRaidTools:AddAltContinue()
	local playerInfo = MorthalinRaidTools:GetPlayerInfo(newAlt);
	if(playerInfo == nil) then
		MorthalinRaidTools:Print("|cffff0000Character does not exist or is not in guild!");
	else
		GuildRosterSetOfficerNote(playerInfo.index, "Alt:" .. playerPick.list[playerPick:GetValue()]:sub(11,-1));
		MorthalinRaidTools:ScheduleTimer(function()
			local indexPlayer = playerPick:GetValue();
			local playerName = playerPick.list[playerPick:GetValue()];
			MorthalinRaidTools:LoadPlayers();
			if(playerPick.list[indexPlayer] == playerName) then
				playerPick:SetValue(indexPlayer);
			elseif(playerPick.list[indexPlayer + 1] == playerName) then
				playerPick:SetValue(indexPlayer + 1);
			elseif(playerPick.list[indexPlayer - 1] == playerName) then
				playerPick:SetValue(indexPlayer - 1);
			end
			MorthalinRaidTools:LoadCharacters();
		end, 1);
	end
end

function MorthalinRaidTools:RemoveAlt()
	if(MorthalinRaidTools:HaveAlts(charPick.list[charPick:GetValue()]:sub(11,-1))) then
		MorthalinRaidTools:Print("|cffff0000Character is main!");
	else
		local playerInfo = MorthalinRaidTools:GetPlayerInfo(charPick.list[charPick:GetValue()]:sub(11,-1));
		GuildRosterSetOfficerNote(playerInfo.index, "");
		MorthalinRaidTools:ScheduleTimer(function()
			local indexPlayer = playerPick:GetValue();
			local playerName = playerPick.list[playerPick:GetValue()];
			MorthalinRaidTools:LoadPlayers();
			if(playerPick.list[indexPlayer] == playerName) then
				playerPick:SetValue(indexPlayer);
			elseif(playerPick.list[indexPlayer + 1] == playerName) then
				playerPick:SetValue(indexPlayer + 1);
			elseif(playerPick.list[indexPlayer - 1] == playerName) then
				playerPick:SetValue(indexPlayer - 1);
			end
			MorthalinRaidTools:LoadCharacters();
		end, 1);
	end
end

function MorthalinRaidTools:ChangeMain()
	if(MorthalinRaidTools:HaveAlts(charPick.list[charPick:GetValue()]:sub(11,-1))) then
		MorthalinRaidTools:Print("|cffff0000Character is already main!");
	else
		oldMain = playerPick.list[playerPick:GetValue()]:sub(11,-1);
		local playerInfo = MorthalinRaidTools:GetPlayerInfo(oldMain);
		newMain = charPick.list[charPick:GetValue()]:sub(11,-1);
		local altInfo = MorthalinRaidTools:GetPlayerInfo(newMain);
		if(playerInfo ~= nil) then
			GuildRosterSetOfficerNote(playerInfo.index, "Alt:" .. newMain);
		else
			self:ConstructConfirm("Main change", "Old main is not in guild or no longer exist. Do you want to change name in history to new main?\r\n(if no, player kills will split in statistics!)", "Main change", 10);
		end
		
		for i = 1, #players[oldMain].alts do
			local altInfo = MorthalinRaidTools:GetPlayerInfo(players[oldMain].alts[i].name);
			if(players[oldMain].alts[i].name == newMain) then
				GuildRosterSetOfficerNote(altInfo.index, "");
			else
				GuildRosterSetOfficerNote(altInfo.index, "Alt:" .. newMain);
			end
		end
		
		MorthalinRaidTools:ScheduleTimer(function()
			local indexPlayer = playerPick:GetValue();
			MorthalinRaidTools:LoadPlayers();
			if((playerPick.list ~= nil and playerPick.list[indexPlayer] ~= nil) and playerPick.list[indexPlayer]:sub(11,-1) or "" == newMain) then
				playerPick:SetValue(indexPlayer);
			elseif((playerPick.list ~= nil and playerPick.list[indexPlayer] ~= nil) and playerPick.list[indexPlayer]:sub(11,-1) or "" == newMain) then
				playerPick:SetValue(indexPlayer + 1);
			elseif((playerPick.list ~= nil and playerPick.list[indexPlayer] ~= nil) and playerPick.list[indexPlayer]:sub(11,-1) or "" == newMain) then
				playerPick:SetValue(indexPlayer - 1);
			end
		MorthalinRaidTools:LoadCharacters();
		end, 1);
	end
end

function MorthalinRaidTools:ChangeNewMainToHistory()	
	for i = 1, #MorthalinRaidTools.db.global.history do
		for j = 1, #MorthalinRaidTools.db.global.history[i].members do
			if(MorthalinRaidTools.db.global.history[i].members[j].name == oldMain) then
				MorthalinRaidTools.db.global.history[i].members[j].name = newMain;
			end
		end
	end
end
