local AceGUI = LibStub("AceGUI-3.0")

local buffsPick, buffName, spellIdEdit, classPick, includePick, excludePick;

function MorthalinRaidTools:ConstructBuffsSettings()
	self.ui.buffsSetting = AceGUI:Create("Frame");
	self.ui.buffsSetting:SetTitle("Buffs settings");
	self.ui.buffsSetting:SetStatusText("");
	self.ui.buffsSetting:SetLayout("Flow");
	self.ui.buffsSetting:SetWidth(445);
	self.ui.buffsSetting:SetHeight(285);
	self.ui.buffsSetting:EnableResize(false);
	self.ui.buffsSetting.frame:EnableMouse(true);
	self.ui.buffsSetting.frame:SetUserPlaced(true);
	
	buffsPick = AceGUI:Create("Dropdown");
	buffsPick:SetLabel("Buffs");
	buffsPick:SetWidth(360);
	buffsPick:SetCallback("OnValueChanged", function(obj, func, value) MorthalinRaidTools:BuffPicked(); end);
	self.ui.buffsSetting:AddChild(buffsPick);
	
	local addBuff = AceGUI:Create("Icon");
	addBuff:SetImage("Interface\\Icons\\spell_chargepositive");
	addBuff:SetWidth(20);
	addBuff:SetHeight(20);
	addBuff:SetImageSize(17,17);
	addBuff:SetCallback("OnClick", function() MorthalinRaidTools:AddBuff() end);
	self.ui.buffsSetting:AddChild(addBuff);
	
	local removeBuff = AceGUI:Create("Icon");
	removeBuff:SetImage("Interface\\Icons\\spell_chargenegative");
	removeBuff:SetWidth(20);
	removeBuff:SetHeight(20);
	removeBuff:SetImageSize(15,15);
	removeBuff:SetCallback("OnClick", function() MorthalinRaidTools:RemoveBuff(buffsPick:GetValue()); end);
	self.ui.buffsSetting:AddChild(removeBuff);
	
	buffName = AceGUI:Create("EditBox");
	buffName:SetLabel("Buff name");
	buffName:SetWidth(200);
	buffName:SetCallback("OnEnterPressed", function(widget, event, text) MorthalinRaidTools:SetBuffName(text); end);
	self.ui.buffsSetting:AddChild(buffName);
	
	spellIdEdit = AceGUI:Create("EditBox");
	spellIdEdit:SetLabel("Spell ID (all if empty)");
	spellIdEdit:SetWidth(200);
	spellIdEdit:SetCallback("OnEnterPressed", function(widget, event, text) MorthalinRaidTools:SetSpellId(text); end);
	self.ui.buffsSetting:AddChild(spellIdEdit);
	
	classPick = AceGUI:Create("Dropdown");
	classPick:SetLabel("Classes");
	classPick:SetWidth(400);
	classPick:SetMultiselect(true);
	classPick:SetCallback("OnValueChanged", function(self, func, key, checked) MorthalinRaidTools:GetPickedClasses(); end);
	self.ui.buffsSetting:AddChild(classPick);
	
	includePick = AceGUI:Create("Dropdown");
	includePick:SetLabel("Include");
	includePick:SetWidth(160);
	includePick:SetCallback("OnValueChanged", function() MorthalinRaidTools:IncludePicked(); end);
	self.ui.buffsSetting:AddChild(includePick);
	
	local addInclude = AceGUI:Create("Icon");
	addInclude:SetImage("Interface\\Icons\\spell_chargepositive");
	addInclude:SetWidth(20);
	addInclude:SetHeight(20);
	addInclude:SetImageSize(15,15);
	addInclude:SetCallback("OnClick", function() MorthalinRaidTools:AddInclude(); end);
	self.ui.buffsSetting:AddChild(addInclude);
	
	local removeInclude = AceGUI:Create("Icon");
	removeInclude:SetImage("Interface\\Icons\\spell_chargenegative");
	removeInclude:SetWidth(20);
	removeInclude:SetHeight(20);
	removeInclude:SetImageSize(15,15);
	removeInclude:SetCallback("OnClick", function() MorthalinRaidTools:RemoveInclude(); end);
	self.ui.buffsSetting:AddChild(removeInclude);
	
	excludePick = AceGUI:Create("Dropdown");
	excludePick:SetLabel("Exclude");
	excludePick:SetWidth(160);
	excludePick:SetCallback("OnValueChanged", function() MorthalinRaidTools:ExcludePicked(); end);
	self.ui.buffsSetting:AddChild(excludePick);
	
	local addExclude = AceGUI:Create("Icon");
	addExclude:SetImage("Interface\\Icons\\spell_chargepositive");
	addExclude:SetWidth(20);
	addExclude:SetHeight(20);
	addExclude:SetImageSize(15,15);
	addExclude:SetCallback("OnClick", function() MorthalinRaidTools:AddExclude(); end);
	self.ui.buffsSetting:AddChild(addExclude);
	
	local removeExclude = AceGUI:Create("Icon");
	removeExclude:SetImage("Interface\\Icons\\spell_chargenegative");
	removeExclude:SetWidth(20);
	removeExclude:SetHeight(20);
	removeExclude:SetImageSize(15,15);
	removeExclude:SetCallback("OnClick", function() MorthalinRaidTools:RemoveExclude(); end);
	self.ui.buffsSetting:AddChild(removeExclude);
	
	includeName = AceGUI:Create("EditBox");
	includeName:SetLabel("");
	includeName:SetWidth(200);
	includeName:SetCallback("OnEnterPressed", function(widget, event, text) MorthalinRaidTools:SetIncludeName(text); end);
	self.ui.buffsSetting:AddChild(includeName);
	
	excludeName = AceGUI:Create("EditBox");
	excludeName:SetLabel("");
	excludeName:SetWidth(200);
	excludeName:SetCallback("OnEnterPressed", function(widget, event, text) MorthalinRaidTools:SetExcludeName(text); end);
	self.ui.buffsSetting:AddChild(excludeName);
	
	MorthalinRaidTools:LoadBuffs();
	MorthalinRaidTools:SetClassesPick();
end

function MorthalinRaidTools:ShowBuffsSettings()
	if(self.ui.buffsSetting:IsVisible() == false) then
		self.ui.buffsSetting:Show();
	end
end

function MorthalinRaidTools:HideBuffsSettings()
	if(self.ui.buffsSetting:IsVisible()) then
		self.ui.buffsSetting:Hide();
	end
end

function MorthalinRaidTools:LoadBuffs()
	local function sortBuffs(a, b)
		return a.name < b.name;
	end
	table.sort(self.db.global.buffs, sortBuffs);
	local index = 1;
	buffsPick:SetValue(nil);
	buffsPick:SetList(nil);
	for ind, data in pairs(self.db.global.buffs) do
		buffsPick:AddItem(index, data.name);
		index = index + 1;
	end
	buffsPick.buffs = self.db.global.buffs;
	MorthalinRaidTools:BuffPicked();
end

function MorthalinRaidTools:SetClassesPick()
	classPick:SetList(nil);
	classPick:AddItem(1, "|cffFF7D0ADruid");
	classPick:AddItem(2, "|cffABD473Hunter");
	classPick:AddItem(3, "|cff40C7EBMage");
	classPick:AddItem(4, "|cffFFFFFFPriest");
	classPick:AddItem(5, "|cffFFF569Rogue");
	classPick:AddItem(6, "|cffF58CBAShaman");
	classPick:AddItem(7, "|cffF58CBAPaladin");
	classPick:AddItem(8, "|cff8787EDWarlock");
	classPick:AddItem(9, "|cffC79C6EWarrior");
end

function MorthalinRaidTools:AddBuff()
	table.insert(self.db.global.buffs, {name = "New buff", index = #self.db.global.buffs, classes = {}, include = {}, exclude = {}, spellId = nil});
	buffsPick:AddItem(#self.db.global.buffs, "New buff");
	buffName:SetText("New buff");
	local text;
	for i, widget in classPick.pullout:IterateItems() do
		if widget.type == "Dropdown-Item-Toggle" then
			widget.value = true;
			widget.check:Show();
			if text then
				text = text..", "..widget:GetText();
			else
				text = widget:GetText();
			end
		end
	end
	classPick:SetText(text);
	buffsPick:SetValue(#self.db.global.buffs);
	MorthalinRaidTools:GetPickedClasses();
end

function MorthalinRaidTools:SetPickedClasses()
	local uniqueClasses = {};
	for i = 1, #self.db.global.buffs[buffsPick:GetValue()].classes do
		uniqueClasses[self.db.global.buffs[buffsPick:GetValue()].classes[i]] = self.db.global.buffs[buffsPick:GetValue()].classes[i];
	end
	local text;
	for i, widget in classPick.pullout:IterateItems() do
		if widget.type == "Dropdown-Item-Toggle" then
			if(uniqueClasses[widget:GetText():sub(11)] ~= nil) then
				widget.value = true;
				widget.check:Show();
				if text then
					text = text..", "..widget:GetText();
				else
					text = widget:GetText();
				end
			else
				widget.value = false;
				widget.check:Hide();
			end
		end
	end
	classPick:SetText(text);
end

function MorthalinRaidTools:GetPickedClasses()
	local classes = {};
	for i, widget in classPick.pullout:IterateItems() do
		if widget.type == "Dropdown-Item-Toggle" then
			if(widget:GetValue()) then
				table.insert(classes, widget:GetText():sub(11));
			end
		end
	end
	self.db.global.buffs[buffsPick:GetValue()].classes = classes;
end

function MorthalinRaidTools:SetBuffName(text)
	self.db.global.buffs[buffsPick:GetValue()].name = text;
	buffsPick.pullout.items[buffsPick:GetValue()]:SetText(text);
	buffsPick.list[buffsPick:GetValue()] = text;
	buffsPick:SetText(text);
end

function MorthalinRaidTools:SetSpellId(text)
	local temp = text:gsub("%D+", "");
	if(#text == #temp) then
		self.db.global.buffs[buffsPick:GetValue()].spellId = #text == 0 and nil or tonumber(text);
	else
		spellIdEdit:SetText(self.db.global.buffs[buffsPick:GetValue()].spellId == nil and "" or tostring(self.db.global.buffs[buffsPick:GetValue()].spellId));
	end
end

function MorthalinRaidTools:RemoveBuff(index)
	table.remove(self.db.global.buffs, index);
	MorthalinRaidTools:LoadBuffs();
end

function MorthalinRaidTools:BuffPicked()
	if(buffsPick:GetValue() ~= nil) then
		buffName:SetText(self.db.global.buffs[buffsPick:GetValue()].name);
		if(self.db.global.buffs[buffsPick:GetValue()].spellId ~= nil) then
			spellIdEdit:SetText(tostring(self.db.global.buffs[buffsPick:GetValue()].spellId));
		else
			spellIdEdit:SetText("");
		end
		MorthalinRaidTools:SetPickedClasses();
	else
		buffName:SetText("");
		classPick:SetValue(nil);
	end
	MorthalinRaidTools:LoadIncludes();
	MorthalinRaidTools:LoadExcludes();
end

function MorthalinRaidTools:LoadIncludes()
	includeName:SetText(nil);
	includePick:SetValue(nil);
	includePick:SetList(nil);
	if(buffsPick:GetValue() ~= nil) then
		for i = 1, #self.db.global.buffs[buffsPick:GetValue()].include do
			includePick:AddItem(i, self.db.global.buffs[buffsPick:GetValue()].include[i]);
		end
	end
end

function MorthalinRaidTools:LoadExcludes()
	excludeName:SetText(nil);
	excludePick:SetValue(nil);
	excludePick:SetList(nil);
	if(buffsPick:GetValue() ~= nil) then
		for i = 1, #self.db.global.buffs[buffsPick:GetValue()].exclude do
			excludePick:AddItem(i, self.db.global.buffs[buffsPick:GetValue()].exclude[i]);
		end
	end
end

function MorthalinRaidTools:AddInclude()
	if(buffsPick:GetValue() ~= nil) then
		table.insert(self.db.global.buffs[buffsPick:GetValue()].include, "New include");
		includePick:AddItem(#self.db.global.buffs[buffsPick:GetValue()].include, "New include");
		includeName:SetText("New include");
		includePick:SetValue(#self.db.global.buffs[buffsPick:GetValue()].include);
	end
end

function MorthalinRaidTools:AddExclude()
	if(buffsPick:GetValue() ~= nil) then
		table.insert(self.db.global.buffs[buffsPick:GetValue()].exclude, "New exclude");
		excludePick:AddItem(#self.db.global.buffs[buffsPick:GetValue()].exclude, "New exclude");
		excludeName:SetText("New exclude");
		excludePick:SetValue(#self.db.global.buffs[buffsPick:GetValue()].exclude);
	end
end

function MorthalinRaidTools:RemoveInclude()
	table.remove(self.db.global.buffs[buffsPick:GetValue()].include, includePick:GetValue());
	MorthalinRaidTools:LoadIncludes();
end

function MorthalinRaidTools:RemoveExclude()
	table.remove(self.db.global.buffs[buffsPick:GetValue()].exclude, includePick:GetValue());
	MorthalinRaidTools:LoadExcludes();
end

function MorthalinRaidTools:IncludePicked()
	includeName:SetText(self.db.global.buffs[buffsPick:GetValue()].include[includePick:GetValue()]);
end

function MorthalinRaidTools:ExcludePicked()
	excludeName:SetText(self.db.global.buffs[buffsPick:GetValue()].exclude[excludePick:GetValue()]);
end

function MorthalinRaidTools:SetIncludeName(text)
	self.db.global.buffs[buffsPick:GetValue()].include[includePick:GetValue()] = text;
	includePick.pullout.items[includePick:GetValue()]:SetText(text);
	includePick.list[includePick:GetValue()] = text;
	includePick:SetText(text);
end

function MorthalinRaidTools:SetExcludeName(text)
	self.db.global.buffs[buffsPick:GetValue()].exclude[excludePick:GetValue()] = text;
	excludePick.pullout.items[excludePick:GetValue()]:SetText(text);
	excludePick.list[excludePick:GetValue()] = text;
	excludePick:SetText(text);
end