local AceGUI = LibStub("AceGUI-3.0")

local editbox = nil;
local btnImport = nil;

local enum = {
	INVITES = 1,
	HISTDATE = 2,
	HISTINST = 3,
	HISTBOSS = 4,
	IMPITEMS = 5,
};
local windowType;

-- History toggle action
-- =========================
function MorthalinRaidTools:HideEditWindow()
	if (MorthalinRaidTools.ui.editFrame:IsVisible()) then
		MorthalinRaidTools.ui.editFrame:Hide();
	end
end

function MorthalinRaidTools:ShowEditWindow(import, winType, data)
	windowType = winType;
	
	if (MorthalinRaidTools.ui.editFrame:IsVisible() == false) then
		MorthalinRaidTools.ui.editFrame:Show();
	end
	
	if(import) then
		MorthalinRaidTools.ui.editFrame:SetHeight(500);
		btnImport.frame:Show();
	else
		MorthalinRaidTools.ui.editFrame:SetHeight(475);
		btnImport.frame:Hide();
	end
	
	if(windowType == enum.HISTBOSS) then
		MorthalinRaidTools:CreateCSVBossBuffs(data);
	elseif(windowType == enum.HISTINST) then
		MorthalinRaidTools:CreateCSVInstancePlayers(data);
	elseif(windowType == enum.HISTDATE) then
		MorthalinRaidTools:CreateCSVDatePlayers(data);
	else
		editbox:SetText("");
	end
end

function MorthalinRaidTools:ConstructEditFrame()
	MorthalinRaidTools.ui.editFrame = AceGUI:Create("Frame");
	MorthalinRaidTools.ui.editFrame:SetTitle("Import/Export");
	MorthalinRaidTools.ui.editFrame:SetStatusText("");
	MorthalinRaidTools.ui.editFrame:SetCallback("OnClose", function(widget) --[[AceGUI:Release(widget)--]] end);
	MorthalinRaidTools.ui.editFrame:SetLayout("Flow");
	MorthalinRaidTools.ui.editFrame:SetWidth(525);
	MorthalinRaidTools.ui.editFrame:SetHeight(475);
	MorthalinRaidTools.ui.editFrame:EnableResize(false);
	
	editbox = AceGUI:Create("MultiLineEditBox");
	editbox:SetLabel("");
	editbox:DisableButton(true);
	editbox:SetWidth(500);
	editbox:SetHeight(400);
	MorthalinRaidTools.ui.editFrame:AddChild(editbox);
	
	btnImport = AceGUI:Create("Button");
	btnImport:SetText("Import");
	btnImport:SetCallback("OnClick", function() self:Import(); end);
	MorthalinRaidTools.ui.editFrame:AddChild(btnImport);
	
	MorthalinRaidTools.ui.editFrame:Hide();
end

function MorthalinRaidTools:Import()
	if(windowType == enum.INVITES) then
		MorthalinRaidTools:LoadInvites();
	elseif(windowType == enum.IMPITEMS) then
		MorthalinRaidTools:LoadItemsImport();
	end
end

function MorthalinRaidTools:CreateCSVBossBuffs(kill)
	local buffsSorted = self.db.global.buffs;
	local text = kill.bossName .. " " .. date("%d.%m.%y %H:%M:%S",kill.killDate);
	local uniqueBuffs = {};
	local alts = {};
	
	if(MorthalinRaidTools.db.global.expBuffsMains) then
		for i=1, GetNumGuildMembers() do
			local name, rank, rankIndex, level, class, zone, note, officernote, online, status = GetGuildRosterInfo(i);
			if(string.sub(officernote, 1, 4) == "Alt:") then
				if(alts[string.sub(name, 1, name:find("-") - 1)] == nil) then
					alts[string.sub(name, 1, name:find("-") - 1)] = string.sub(officernote, 5, -1);
				end
			end
		end
	end
	
	for i = 1, #kill.members do
		if(kill.members[i].group ~= 0 and kill.members[i].buffs ~= nil) then
			for name, value in pairs(kill.members[i].buffs) do
				local clearName = name:gsub("%s+", "");
				if(uniqueBuffs[clearName] == nil) then
					for j = 1, #value do
						uniqueBuffs[clearName] = {};
						uniqueBuffs[clearName][tostring(value[j].spellId)] = value[j].spellId;
					end
				else
					for j = 1, #value do
						if(uniqueBuffs[clearName][tostring(value[j].spellId)] == nil) then
							uniqueBuffs[clearName][tostring(value[j].spellId)] = value[j].spellId;
						end
					end
				end
			end
		end
	end
	
	local function sortBuffs(a, b)
		return a.name < b.name;
	end
	table.sort(buffsSorted, sortBuffs);
	table.sort(kill.members, sortBuffs);
	
	--NAMES OF BUFFS
	for i = 1, #buffsSorted do
		if(uniqueBuffs[buffsSorted[i].name:gsub("%s+", "")] ~= nil) then
			if(buffsSorted[i].spellId == nil and MorthalinRaidTools.db.global.expBuffsMerge == false) then
				for index, spellId in pairs(uniqueBuffs[buffsSorted[i].name:gsub("%s+", "")]) do
					if(MorthalinRaidTools:CheckBuffNameAndId(buffsSorted[i].name:gsub("%s+", ""), spellId) and (buffsSorted[i].spellId == nil or buffsSorted[i].spellId == spellId or spellId == 0)) then
						text = text .. ";" .. buffsSorted[i].name;
					end
				end
			elseif(buffsSorted[i].spellId ~= nil) then
				text = text .. ";" .. buffsSorted[i].name;
			else
				text = text .. ";" .. buffsSorted[i].name;
			end
		else
			text = text .. ";" .. buffsSorted[i].name;
		end
	end
	text = text .. "\r\n";
	--IDs OF BUFFS
	text = text .. "Spell IDs (0=enchant/not used,-1=merged)";
	for i = 1, #buffsSorted do
		buffsSorted[i].spellIds = {};
		if(uniqueBuffs[buffsSorted[i].name:gsub("%s+", "")] ~= nil) then
			if(buffsSorted[i].spellId ~= nil or (buffsSorted[i].spellId == nil and MorthalinRaidTools.db.global.expBuffsMerge == false)) then
				local found = false;
				for index, spellId in pairs(uniqueBuffs[buffsSorted[i].name:gsub("%s+", "")]) do
					if(MorthalinRaidTools:CheckBuffNameAndId(buffsSorted[i].name:gsub("%s+", ""), spellId) and (buffsSorted[i].spellId == nil or buffsSorted[i].spellId == spellId or spellId == 0)) then
						text = text .. ";" .. spellId;
						table.insert(buffsSorted[i].spellIds, spellId);
						found = true;
					end
				end
				if(buffsSorted[i].spellId ~= nil and found == false) then
					text = text .. ";0";
					table.insert(buffsSorted[i].spellIds, buffsSorted[i].spellId);
				end
			else
				local countSpellIds = 0;
				local tempSpellID = -1;
				for index, spellId in pairs(uniqueBuffs[buffsSorted[i].name:gsub("%s+", "")]) do
					countSpellIds = countSpellIds + 1;
					tempSpellID = spellId;
				end
				if(countSpellIds > 1) then
					text = text .. ";-1";
					table.insert(buffsSorted[i].spellIds, -1);
				else
					text = text .. ";" .. tempSpellID;
					table.insert(buffsSorted[i].spellIds, tempSpellID);
				end
			end
		else
			text = text .. ";0";
			table.insert(buffsSorted[i].spellIds, 0);
		end
	end
	text = text .. "\r\n";
	
	for i = 1, #kill.members do
		if(alts[kill.members[i].name] ~= nil) then
			text = text .. alts[kill.members[i].name];
		else
			text = text .. kill.members[i].name;
		end
		local playerSorted =  {};
		if(kill.members[i].buffs ~= nil) then
			for name, value in pairs(kill.members[i].buffs) do
				for j = 1, #value do
					table.insert(playerSorted,{name = name:gsub("%s+", ""), spellId = value[j].spellId});
				end
			end
			table.sort(playerSorted, sortBuffs);
		end
		
		for j = 1, #buffsSorted do
			for k = 1, #buffsSorted[j].spellIds do
				text = text .. ";";
				if(kill.members[i].group == 0) then
					text = text .. "B";
				elseif(kill.members[i].group ~= 0 and MorthalinRaidTools:CheckBuffOnPlayer(buffsSorted[j], playerSorted, kill.members[i].class, kill.members[i].name, k)) then
					text = text .. "1";
				elseif(kill.members[i].group ~= 0) then
					text = text .. "0";
				end
			end
		end
		text = text .. "\r\n";
	end
	
	editbox:SetText(text);
	editbox:HighlightText(0, #text);
	editbox:SetFocus();
end

function MorthalinRaidTools:CheckBuffOnPlayer(setting, buffs, class, name, spellIdIndex)
	local ret = false;
	for i = 1, #buffs do
		if(setting.name:gsub("%s+", "") == buffs[i].name:gsub("%s+", "") and (setting.spellIds[spellIdIndex] == buffs[i].spellId or (setting.spellIds[spellIdIndex] == -1 and MorthalinRaidTools.db.global.expBuffsMerge and setting.spellId == nil)) and (setting.spellId == nil or setting.spellId == setting.spellIds[spellIdIndex] or setting.spellIds[spellIdIndex] == 0)) then
			for j = 1, #setting.classes do
				if(class == setting.classes[j]) then
					ret = true;
					j = #buffs[i];
				end
			end
			if(ret == false) then
				for j = 1, #setting.include do
					if(name == setting.include[j]) then
						ret = true;
					end
				end
			end
			if(ret == true) then
				for j = 1, #setting.exclude do
					if(name == setting.exclude[j]) then
						ret = false;
					end
				end
			end
			i = #buffs;
		end
	end
	return ret;
end

function MorthalinRaidTools:CheckBuffNameAndId(name, spellId)
	for i = 1, #self.db.global.buffs do
		if(self.db.global.buffs[i].name:gsub("%s+", "") == name:gsub("%s+", "") and (self.db.global.buffs[i].spellId == nil or self.db.global.buffs[i].spellId == spellId or spellId == 0)) then
			return true;
		end
	end
	return false;
end

function MorthalinRaidTools:CreateCSVInstancePlayers(instance)
	local rows = self.db.global.history;
	local kills = {};
	local players = {};
	local alts = {};
	local text = instance.name;
	
	if(MorthalinRaidTools.db.global.expBuffsMains) then
		for i=1, GetNumGuildMembers() do
			local name, rank, rankIndex, level, class, zone, note, officernote, online, status = GetGuildRosterInfo(i);
			if(string.sub(officernote, 1, 4) == "Alt:") then
				if(alts[string.sub(name, 1, name:find("-") - 1)] == nil) then
					alts[string.sub(name, 1, name:find("-") - 1)] = string.sub(officernote, 5, -1);
				end
			end
		end
	end
	
	for i = 1, #rows do
		for j = 1, #self.db.global.history[i].members do
			local tempName = {};
			if(alts[self.db.global.history[i].members[j].name] ~= nil) then
				tempName = alts[self.db.global.history[i].members[j].name];
			else
				tempName = self.db.global.history[i].members[j].name;
			end
			if(players[tempName] == nil) then
				players[tempName] = {name = tempName};
			end
		end
		
		if (date("%H",rows[i].killDate) == "00" or date("%H",rows[i].killDate) == "01") then
			dateTemp = date("%d.%m.%Y",(rows[i].killDate - 7200))
		else
			dateTemp = date("%d.%m.%Y",rows[i].killDate)
		end
		if (dateTemp == date("%d.%m.%Y",instance.start) and rows[i].zone == instance.name and rows[i].instanceID == instance.instanceID) then
			table.insert(kills, rows[i]);
		end
	end	
	
	local function sortKills(a, b)
		return a.killDate < b.killDate;
	end
	table.sort(kills, sortKills);
	
	if(MorthalinRaidTools.db.global.expBosses) then
		for i = 1, #kills do
			text = text .. ";" .. kills[i].bossName;
		end
	end
	if(MorthalinRaidTools.db.global.expAll) then
		text = text .. ";All\r\n";
	else
		text = text .. "\r\n";
	end
	
	players = self:ConvertStringIndex(players);
	local function sortPlayers(a, b)
		return a.index < b.index;
	end
	table.sort(players, sortPlayers);
	
	for i = 1, #players do
		local count = 0;
		text = text .. players[i].name;
		for j = 1, #kills do
			local found = false;
			for k = 1, #kills[j].members do
				if(kills[j].members[k].name  == players[i].name or alts[kills[j].members[k].name] == players[i].name) then
					if(MorthalinRaidTools.db.global.expBosses) then
						if(kills[j].members[k].group == 0) then
							text = text .. ";B";
						else
							text = text .. ";1";
						end
					end
					count = count + 1;
					found = true;
					k = #kills[j].members;
				end
			end
			if(found == false and MorthalinRaidTools.db.global.expBosses) then
				text = text .. ";0";
			end
		end
		if(MorthalinRaidTools.db.global.expAll) then
			text = text .. ";" .. string.gsub(tostring(self:Round((count / #kills), 2)), "%.", ",") .. "\r\n";
		else
			text = text .. "\r\n";
		end
	end
	
	editbox:SetText(text);
	editbox:HighlightText(0, #text);
	editbox:SetFocus();
end

function MorthalinRaidTools:CreateCSVDatePlayers(raidDate)
	local rows = self.db.global.history;
	local kills = {};
	local players = {};
	local alts = {};
	local text = raidDate;
	local raids = {};
	
	if(MorthalinRaidTools.db.global.expBuffsMains) then
		for i=1, GetNumGuildMembers() do
			local name, rank, rankIndex, level, class, zone, note, officernote, online, status = GetGuildRosterInfo(i);
			if(string.sub(officernote, 1, 4) == "Alt:") then
				if(alts[string.sub(name, 1, name:find("-") - 1)] == nil) then
					alts[string.sub(name, 1, name:find("-") - 1)] = string.sub(officernote, 5, -1);
				end
			end
		end
	end
	
	for i = 1, #rows do
		for j = 1, #self.db.global.history[i].members do
			local tempName = {};
			if(alts[self.db.global.history[i].members[j].name] ~= nil) then
				tempName = alts[self.db.global.history[i].members[j].name];
			else
				tempName = self.db.global.history[i].members[j].name;
			end
			if(players[tempName] == nil) then
				players[tempName] = {name = tempName, counts = {}};
			end
		end
		
		if (date("%H",rows[i].killDate) == "00" or date("%H",rows[i].killDate) == "01") then
			dateTemp = date("%d.%m.%Y",(rows[i].killDate - 7200))
		else
			dateTemp = date("%d.%m.%Y",rows[i].killDate)
		end
		if (dateTemp == raidDate) then
			table.insert(kills, rows[i]);
			
			if(raids[rows[i].zone] == nil) then
				raids[rows[i].zone] = 1;
			else
				raids[rows[i].zone] = raids[rows[i].zone] + 1;
			end
		end
	end	
	
	local function sortKills(a, b)
		return a.killDate < b.killDate;
	end
	table.sort(kills, sortKills);
	
	if(MorthalinRaidTools.db.global.expBosses) then
		for i = 1, #kills do
			text = text .. ";" .. kills[i].bossName;
		end
	end
	
	if(MorthalinRaidTools.db.global.expInstance) then
		for name, count in pairs(raids) do
			text = text .. ";" .. name;
		end
	end
	
	if(MorthalinRaidTools.db.global.expAll) then
		text = text .. ";All\r\n";
	else
		text = text .. "\r\n";
	end
	
	players = self:ConvertStringIndex(players);
	local function sortPlayers(a, b)
		return a.index < b.index;
	end
	table.sort(players, sortPlayers);
	
	for i = 1, #players do
		text = text .. players[i].name;
		for j = 1, #kills do
			local found = false;
			for k = 1, #kills[j].members do
				if(kills[j].members[k].name  == players[i].name or alts[kills[j].members[k].name] == players[i].name) then
					if(MorthalinRaidTools.db.global.expBosses) then
						if(kills[j].members[k].group == 0) then
							text = text .. ";B";
						else
							text = text .. ";1";
						end
					end
					if(players[i].counts[kills[j].zone] == nil) then
						players[i].counts[kills[j].zone] = 1;
					else
						players[i].counts[kills[j].zone] = players[i].counts[kills[j].zone] + 1;
					end
					found = true;
					k = #kills[j].members;
				end
			end
			if(found == false and MorthalinRaidTools.db.global.expBosses) then
				text = text .. ";0";
			end
		end
		local count, countAll = 0, 0;
		for name, counts in pairs(raids) do
			count = count + (players[i].counts[name] == nil and 0 or players[i].counts[name]);
			countAll = countAll + counts;
			if(MorthalinRaidTools.db.global.expInstance) then
				text = text .. ";" .. string.gsub(tostring(self:Round((players[i].counts[name] == nil and 0 or (players[i].counts[name] / counts)), 2)), "%.", ",");
			end
		end
		if(MorthalinRaidTools.db.global.expAll) then
			text = text .. ";" .. string.gsub(tostring(self:Round((count / countAll), 2)), "%.", ",") .. "\r\n";
		else
			text = text .. "\r\n";
		end
	end
	
	editbox:SetText(text);
	editbox:HighlightText(0, #text);
	editbox:SetFocus();
end

function MorthalinRaidTools:LoadInvites()
	local failed = false;
	local name = MorthalinRaidTools:FindName();
	if(name ~= nil or name == "") then
		name = MorthalinRaidTools:ClearName(name);
	else
		failed = true;
	end
	local raidDate = MorthalinRaidTools:FindDate();
	if(raidDate == nil) then
		failed = true;
	end
	if(failed) then
		MorthalinRaidTools:Print("|cffff0000Wrong data format!");
	else
		local raid = {name = name, raidDate = raidDate, members = {}};
		if(MorthalinRaidTools:CheckRaidSchedule(raid.name, raid.raidDate) == false) then
			MorthalinRaidTools:FindRole("Tank", raid.members);
			for name, data in pairs(MorthalinRaidTools.classColors) do
				MorthalinRaidTools:FindRole(name, raid.members);
			end
			
			table.insert(MorthalinRaidTools.db.global.schedule, raid);
		else
			MorthalinRaidTools:Print("|cffff0000Raid with name '" .. raid.name .. "' was already loaded for day " .. date("%d.%m.%Y", raid.raidDate));
		end
		MorthalinRaidTools:LoadScheduleRaids();
		MorthalinRaidTools:HideEditWindow();
	end
end

function MorthalinRaidTools:FindName()
	local text = editbox:GetText();
	local res = nil;
	local found = false;
	while (found == false and text:find("\n") ~= nil) do
		if(text:find("Raid Leader:") == 1) then
			text = text:sub(text:find("\n") + 1, -1);
			res = text:sub(0, text:find("\n") - 1);
			found = true;
		else
			text = text:sub(text:find("\n") + 1, -1);
		end
	end
	return res;
end

function MorthalinRaidTools:ClearName(name)
	local res = "";
	local text = name;
	
	while(text:find(":.+:") ~= nil) do
		local startI = text:find(":");
		local endI = text:find(":", 2);
		local symbol = text:sub(startI + 1, endI - 1):gsub("_", "");
		if(symbol == "empty") then
			symbol = " ";
		end
		if(symbol == "plus") then
			symbol = "+";
		end
		res = res .. symbol;
		text = text:sub(endI + 2);
	end
	
	return res;
end

function MorthalinRaidTools:FindDate()
	local text = editbox:GetText();
	local dateText = nil;
	local timeText = nil;
	local res = nil;
	local startI, endI = text:find(":CMcalendar[^:]*:[^\n]*\n");
	if(endI ~= nil) then
		dateText = text:sub(startI, endI - 1);
		dateText = dateText:sub(dateText:find("(%d+)-(%d+)-(%d+)"));
	end
	
	startI, endI = text:find(":CMclock[^:]*:[^\n]*\n");
	if(endI ~= nil) then
		timeText = text:sub(startI, endI - 1);
		timeText = timeText:sub(timeText:find("%d%d:%d%d"));
	end
	
	if(dateText ~= nil and timeText ~= nil) then
		local day, month, year = dateText:match('(%d+)-(%d+)-(%d+)');
		local hour, minute = timeText:match('(%d+):(%d+)');
		res = time{year=year, month=month, day=day, hour=hour, min=minute};
	end
	
	return res;
end

function MorthalinRaidTools:FindRole(roleName, players)
	local text = editbox:GetText();
	local count = 0;
	
	local startI, endI = text:find(":" .. roleName .. ": " .. roleName .. "[^\n]*\n");
	if(endI ~= nil) then
		local temp = text:sub(startI, endI);
		temp = temp:sub(temp:find("%(%d+%)"))
		count = tonumber(temp:sub(2, -2));
		temp = "";
		for i = 1, count do
			temp = temp .. "[^\n]*\n";
		end
		text = text:sub(endI + 1);
		if(text:find(temp) ~= nil) then
			text = text:sub(text:find(temp));
		else
			text = "";
		end
		while(text:find("\n") ~= nil) do
			if(text:find("%d [^\n]*\n") ~= nil) then
				temp = text:sub(text:find("%d [^\n]*\n"));
				temp = temp:sub(3, -2);
				local playerInfo = MorthalinRaidTools:GetPlayerInfo(temp);
				local playerTemp = {
					name = nil,
					class = nil,
					color = nil
				}
				if(playerInfo ~= nil) then
					if((roleName == playerInfo.class or (roleName == "Tank" and (playerInfo.class == "Warrior" or playerInfo.class == "Paladin" or playerInfo.class == "Druid")))) then
						playerTemp.name = playerInfo.name:sub(1, playerInfo.name:find("-") - 1);
						playerTemp.class = playerInfo.class;
					else
						playerTemp.name, playerTemp.class = MorthalinRaidTools:FindClass(temp, roleName);
					end
				else
					playerTemp.name = "(!)" .. temp;
					playerTemp.class = roleName == "Tank" and "Warrior" or roleName;
				end
				playerTemp.color = MorthalinRaidTools:GetClassColor(playerTemp.class);
				table.insert(players, playerTemp);
			end
			text = text:sub(text:find("\n") + 1);
		end
	end
end

function MorthalinRaidTools:FindClass(nameAlt, classAlt)
	local temp;
	if(string.len(nameAlt) > 1) then
		temp = string.upper(string.sub(nameAlt, 1, 1))..string.lower(string.sub(nameAlt, 2));
	elseif(string.len(nameAlt) == 1) then
		temp = string.upper(string.sub(nameAlt, 1, 1));
	end
	for i=1, GetNumGuildMembers() do
		local name, rank, rankIndex, level, class, zone, note, officernote, online, status = GetGuildRosterInfo(i);
		if(MorthalinRaidTools.db.global.impMaxLvlAlts == false or level == MorthalinRaidTools.db.global.maxLevelOnExpansion[GetAccountExpansionLevel()]) then
			if(officernote == "Alt:" .. temp) then
				if(classAlt == "Tank") then
					if(class == "Warrior" or class == "Paladin" or class == "Druid") then
						return name:sub(1, name:find("-") - 1), class;
					end
				elseif(classAlt == class) then
					return name:sub(1, name:find("-") - 1), class;
				end
			end
		end
	end
	return "(!)" .. temp, classAlt;
end

function MorthalinRaidTools:LoadItemsImport()
	local failed = false;
	local itemNames = MorthalinRaidTools:FindRowByTemplImp("Name");
	local itemIcons = MorthalinRaidTools:FindRowByTemplImp("Icon");
	local players = MorthalinRaidTools:FindPlayerCountsItemImp();
	if(#itemNames == 0 or #itemNames ~= #itemIcons) then
		failed = true;
	else
		for i = 1, #itemNames do
			if(itemNames[i] == nil or #itemNames[i] == 0) then
				failed = true;
			end
		end
		for i = 1, #itemIcons do
			if(itemIcons[i] == nil or #itemIcons[i] == 0) then
				failed = true;
			end
		end
		if(failed == false) then
			for name, counts in pairs(players) do
				if(#counts ~= #itemNames) then
					failed = true;
				end
			end
		end
	end
	
	if(failed) then
		MorthalinRaidTools:Print("|cffff0000Wrong data format!");
	else
		for i = 1, #itemNames do
			MorthalinRaidTools:Print((#itemIcons[i] == 0 and "nil" or itemIcons[i]));
			self.db.global.itemCounter[itemNames[i]] = {players = {}, icon = itemIcons[i]};
			for name, counts in pairs(players) do
				self.db.global.itemCounter[itemNames[i]].players[name] = (#counts[i] == 0 and 0 or tonumber(counts[i]));
			end
		end
		MorthalinRaidTools:Print("Item data was imported.");
		MorthalinRaidTools:HideEditWindow();
	end
end

function MorthalinRaidTools:FindRowByTemplImp(templ)
	local text = editbox:GetText();
	local res = {};
	local found = false;
	while (found == false and text:find("\n") ~= nil) do
		if(text:find(templ) == 1) then
			text = text:sub(#templ + 2, text:find("\n") - 1);
			while(text:find(";") ~= nil) do
				local temp = text:sub(1, text:find(";") - 1)
				table.insert(res, temp);
				text = text:sub(text:find(";") + 1, -1);
			end
			table.insert(res, text);
			found = true;
		else
			text = text:sub(text:find("\n") + 1, -1);
		end
	end
	return res;
end

function MorthalinRaidTools:FindPlayerCountsItemImp()
	local text = editbox:GetText();
	local res = {};
	trash = text:sub(text:find("^[^\n]+\n[^\n]+\n"));
	text = text:sub(#trash + 1, -1);
	while (text:find("\n") ~= nil) do
		local name;
		parts = {strsplit(";", text:sub(1, text:find("\n") - 1))};
		for i = 1, #parts do
			if(i == 1) then
				res[parts[i]] = {};
				name = parts[i];
			else
				table.insert(res[name], parts[i]);
			end
		end
		text = text:sub(text:find("\n") + 1, -1);
	end
	return res;
end