local AceGUI = LibStub("AceGUI-3.0")

local maxWidth;

function MorthalinRaidTools:ConstructIconTable(object, data, buffTime)
	if(self.ui.iconTable ~= nil) then
		self.ui.iconTable:Hide();
	end
	self.ui.iconTable = nil;
	if(self.ui.iconTable == nil) then
		local FrameBackdrop = {
			bgFile = "Interface\\DialogFrame\\UI-DialogBox-Background",
			edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
			tile = true, tileSize = 32, edgeSize = 16,
			insets = { left = 5, right = 5, top = 5, bottom = 5 }
		}
		
		self.ui.iconTable = CreateFrame("Frame",nil,object);
		self.ui.iconTable:SetFrameStrata("TOOLTIP");
		self.ui.iconTable:SetWidth(130);
		self.ui.iconTable:SetHeight(100);
		maxWidth = 130;

		self.ui.iconTable:EnableMouse(true);
		self.ui.iconTable:SetMovable(false);
		self.ui.iconTable:SetResizable(false);
		self.ui.iconTable:SetBackdrop(FrameBackdrop);
		self.ui.iconTable:SetBackdropColor(0, 0, 0, 1);
		self.ui.iconTable:SetToplevel(true);
		
	end
	self:SetIconTableRows(data, buffTime);
	self.ui.iconTable:SetPoint("TOPLEFT", (self.ui.iconTable:GetWidth() * (-1)), 0);
end

function MorthalinRaidTools:SetIconTableIcons(data, top)
	local index = 1;
	local left = 112;
	local buffsSorted = {};
	if(data.buffs ~= nil) then
		for name, value in pairs(data.buffs) do
			table.insert(buffsSorted,{name = name, data = value});
		end
		local function sortBuffs(a, b)
			return a.name < b.name;
		end
		table.sort(buffsSorted, sortBuffs);
		for i = 1, #buffsSorted do
			for j = 1, #buffsSorted[i].data do
				if(MorthalinRaidTools.db.global.showBuffsAll or MorthalinRaidTools:CheckSettings(buffsSorted[i], data.class, data.name, j)) then
					local iconGUI = CreateFrame("BUTTON", nil, self.ui.iconTable, "SecureHandlerClickTemplate");
					iconGUI:SetSize(16, 16);
					iconGUI:SetPoint("TOPLEFT", left, top * (-1));
					
					local image = iconGUI:CreateTexture(nil,"BACKGROUND",nil,-6);
					image:SetTexture(buffsSorted[i].data[j].icon);
					image:SetTexCoord(0, 1, 0, 1);
					image:SetAllPoints(iconGUI);
					iconGUI.name = buffsSorted[i].name;
					iconGUI:SetScript("OnEnter", function()
						GameTooltip:SetOwner(iconGUI,"ANCHOR_LEFT");
						GameTooltip:AddLine(buffsSorted[i].name);
						GameTooltip:Show();
					end);
					iconGUI:SetScript("OnLeave", function() GameTooltip:Hide(); end);
					left = left + 17;
				end
			end
		end
	end
	if(maxWidth < left) then
		maxWidth = left;
		self.ui.iconTable:SetWidth(left + 4);
	end
end

function MorthalinRaidTools:SetIconTableRows(data, buffTime)
	local top = 7;
	local index = 1;
	if(buffTime ~= nil) then
		local row = CreateFrame("Frame", nil, self.ui.iconTable);
		row:SetWidth(120);
		row:SetHeight(10);
		row:SetPoint("TOPLEFT", 7, top * (-1));
		row.text = row:CreateFontString(nil, "ARTWORK");
		row.text:SetFont(GameFontNormal:GetFont());
		row.text:SetJustifyH("LEFT");
		row.text:SetJustifyV("TOP");
		row.text:SetText(date("%d.%m.%y %H:%M:%S",buffTime));
		row.text:SetAllPoints(row);
		top = top + 17;
	end
	for i = 1, #data do
		if(data[i].group ~= 0) then
			local row = CreateFrame("Frame", nil, self.ui.iconTable);
			row:SetWidth(100);
			row:SetHeight(10);
			row:SetPoint("TOPLEFT", 7, top * (-1));
			row.text = row:CreateFontString(nil, "ARTWORK");
			row.text:SetFont(GameFontNormal:GetFont());
			row.text:SetJustifyH("LEFT");
			row.text:SetJustifyV("TOP");
			row.text:SetText("|cff" .. data[i].color .. data[i].name);
			row.text:SetAllPoints(row);
			MorthalinRaidTools:SetIconTableIcons(data[i], top);
			top = top + 17;
			index = index + 1;
		end
	end
	self.ui.iconTable:SetHeight(top + 4);
end

function MorthalinRaidTools:CheckSettings(buff, class, name, index)
	local ret = false;
	for i = 1, #self.db.global.buffs do
		if(buff.name:gsub("%s+", "") == self.db.global.buffs[i].name:gsub("%s+", "") and (buff.data[index].spellId == 0 or self.db.global.buffs[i].spellId == nil or buff.data[index].spellId == self.db.global.buffs[i].spellId)) then
			for j = 1, #self.db.global.buffs[i].classes do
				if(class == self.db.global.buffs[i].classes[j]) then
					ret = true;
					j = #self.db.global.buffs[i];
				end
			end
			if(ret == false) then
				for j = 1, #self.db.global.buffs[i].include do
					if(name == self.db.global.buffs[i].include[j]) then
						ret = true;
					end
				end
			end
			if(ret == true) then
				for j = 1, #self.db.global.buffs[i].exclude do
					if(name == self.db.global.buffs[i].exclude[j]) then
						ret = false;
					end
				end
			end
			i = #self.db.global.buffs;
		end
	end
	return ret;
end
