local AceGUI = LibStub("AceGUI-3.0")

local itemsPick, itemName, iconIdEdit, iconIdPic, playersPick, playerName, itemCount;

function MorthalinRaidTools:ConstructItemSettings()
	self.ui.itemsSetting = AceGUI:Create("Frame");
	self.ui.itemsSetting:SetTitle("Item settings");
	self.ui.itemsSetting:SetStatusText("");
	self.ui.itemsSetting:SetLayout("Flow");
	self.ui.itemsSetting:SetWidth(445);
	self.ui.itemsSetting:SetHeight(255);
	self.ui.itemsSetting:EnableResize(false);
	self.ui.itemsSetting.frame:EnableMouse(true);
	self.ui.itemsSetting.frame:SetUserPlaced(true);
	
	itemsPick = AceGUI:Create("Dropdown");
	itemsPick:SetLabel("Items");
	itemsPick:SetWidth(360);
	itemsPick:SetCallback("OnValueChanged", function(obj, func, value) MorthalinRaidTools:ItemPicked(); end);
	self.ui.itemsSetting:AddChild(itemsPick);
	
	local addItem = AceGUI:Create("Icon");
	addItem:SetImage("Interface\\Icons\\spell_chargepositive");
	addItem:SetWidth(20);
	addItem:SetHeight(20);
	addItem:SetImageSize(17,17);
	addItem:SetCallback("OnClick", function() MorthalinRaidTools:AddItem(); end);
	self.ui.itemsSetting:AddChild(addItem);
	
	local removeItem = AceGUI:Create("Icon");
	removeItem:SetImage("Interface\\Icons\\spell_chargenegative");
	removeItem:SetWidth(20);
	removeItem:SetHeight(20);
	removeItem:SetImageSize(15,15);
	removeItem:SetCallback("OnClick", function() MorthalinRaidTools:RemoveSelectedItem(); end);
	self.ui.itemsSetting:AddChild(removeItem);
	
	itemName = AceGUI:Create("EditBox");
	itemName:SetLabel("Item name");
	itemName:SetWidth(200);
	itemName:SetCallback("OnEnterPressed", function(widget, event, text) MorthalinRaidTools:SetItemName(text); end);
	self.ui.itemsSetting:AddChild(itemName);
	
	iconIdEdit = AceGUI:Create("EditBox");
	iconIdEdit:SetLabel("Icon ID");
	iconIdEdit:SetWidth(180);
	iconIdEdit:SetCallback("OnEnterPressed", function(widget, event, text) MorthalinRaidTools:SetIconId(text); end);
	self.ui.itemsSetting:AddChild(iconIdEdit);
	
	iconIdPic = AceGUI:Create("Icon");
	iconIdPic:SetWidth(20);
	iconIdPic:SetHeight(20);
	iconIdPic:SetImageSize(20,20);
	self.ui.itemsSetting:AddChild(iconIdPic);
	
	playersPick = AceGUI:Create("Dropdown");
	playersPick:SetLabel("Players");
	playersPick:SetWidth(360);
	playersPick:SetCallback("OnValueChanged", function(obj, func, value) MorthalinRaidTools:PlayerPicked(); end);
	self.ui.itemsSetting:AddChild(playersPick);
	
	local addPlayer = AceGUI:Create("Icon");
	addPlayer:SetImage("Interface\\Icons\\spell_chargepositive");
	addPlayer:SetWidth(20);
	addPlayer:SetHeight(20);
	addPlayer:SetImageSize(17,17);
	addPlayer:SetCallback("OnClick", function() MorthalinRaidTools:AddPlayerToItem(); end);
	self.ui.itemsSetting:AddChild(addPlayer);
	
	local removePlayer = AceGUI:Create("Icon");
	removePlayer:SetImage("Interface\\Icons\\spell_chargenegative");
	removePlayer:SetWidth(20);
	removePlayer:SetHeight(20);
	removePlayer:SetImageSize(15,15);
	removePlayer:SetCallback("OnClick", function() MorthalinRaidTools:RemovePlayerFromItem(); end);
	self.ui.itemsSetting:AddChild(removePlayer);
	
	playerName = AceGUI:Create("EditBox");
	playerName:SetLabel("Player name");
	playerName:SetWidth(300);
	playerName:SetCallback("OnEnterPressed", function(widget, event, text) MorthalinRaidTools:SetPlayerName(text); end);
	playerName:SetCallback("OnTextChanged", function(widget, event, text)
		if(string.len(text) ~= 0) then
			self:ConstructWhisperer(playerName, text, 6);
		else
			self:HideWhisperer();
		end
	end)
	self.ui.itemsSetting:AddChild(playerName);
	
	itemCount = AceGUI:Create("EditBox");
	itemCount:SetLabel("Count");
	itemCount:SetWidth(100);
	itemCount:SetCallback("OnEnterPressed", function(widget, event, text) MorthalinRaidTools:SetItemCount(text); end);
	self.ui.itemsSetting:AddChild(itemCount);
	
	MorthalinRaidTools:LoadItems();
end

function MorthalinRaidTools:ShowItemsSetting()
	if(self.ui.itemsSetting:IsVisible() == false) then
		self.ui.itemsSetting:Show();
	end
end

function MorthalinRaidTools:HideItemsSetting()
	if(self.ui.itemsSetting:IsVisible()) then
		self.ui.itemsSetting:Hide();
	end
end

function MorthalinRaidTools:LoadItems()
	local itemNames = {};
	for name, data in pairs(self.db.global.itemCounter) do
		table.insert(itemNames, name);
	end
	local function sortNames(a, b)
		return a < b;
	end
	table.sort(itemNames, sortNames);
	itemsPick:SetValue(nil);
	itemsPick:SetList(nil);
	for i = 1, #itemNames do
		itemsPick:AddItem(i, itemNames[i]);
	end
	itemsPick.itemNames = itemNames;
	MorthalinRaidTools:ItemPicked();
end

function MorthalinRaidTools:AddItem()
	self.db.global.itemCounter["New item"] = {players = {}, icon = nil};
	itemsPick:AddItem(#itemsPick.list + 1, "New item");
	itemName:SetText("New item");
	itemsPick:SetValue(#itemsPick.list);
end

function MorthalinRaidTools:SetItemName(text)
	if(MorthalinRaidTools.db.global.itemCounter[text] == nil) then
		MorthalinRaidTools.db.global.itemCounter[text] = MorthalinRaidTools.db.global.itemCounter[itemsPick.list[itemsPick:GetValue()]];
		MorthalinRaidTools.db.global.itemCounter[itemsPick.list[itemsPick:GetValue()]] = nil;
		itemsPick.pullout.items[itemsPick:GetValue()]:SetText(text);
		itemsPick.list[itemsPick:GetValue()] = text;
		itemsPick:SetText(text);
	else
		message("Name of item '" .. text .. "' was already used!");
		itemName:SetText(itemsPick.list[itemsPick:GetValue()]);
	end
end

function MorthalinRaidTools:SetIconId(text)
	local temp = text:gsub("%D+", "");
	if(#text == #temp) then
		MorthalinRaidTools.db.global.itemCounter[itemsPick.list[itemsPick:GetValue()]].icon = #text == 0 and nil or tonumber(text);
		iconIdPic:SetImage(tonumber(text));
	else
		iconIdEdit:SetText(MorthalinRaidTools.db.global.itemCounter[itemsPick.list[itemsPick:GetValue()]].icon == nil and "" or tostring(MorthalinRaidTools.db.global.itemCounter[itemsPick.list[itemsPick:GetValue()]].icon));
	end
end

function MorthalinRaidTools:RemoveSelectedItem()
	MorthalinRaidTools.db.global.itemCounter[itemsPick.list[itemsPick:GetValue()]] = nil;
	MorthalinRaidTools:LoadItems();
end

function MorthalinRaidTools:ItemPicked()
	if(itemsPick:GetValue() ~= nil) then
		itemName:SetText(itemsPick.list[itemsPick:GetValue()]);
		if(self.db.global.itemCounter[itemsPick.list[itemsPick:GetValue()]].icon ~= nil) then
			iconIdEdit:SetText(tostring(self.db.global.itemCounter[itemsPick.list[itemsPick:GetValue()]].icon));
			iconIdPic:SetImage(self.db.global.itemCounter[itemsPick.list[itemsPick:GetValue()]].icon);
		else
			iconIdEdit:SetText("");
			iconIdPic:SetImage(nil);
		end
		MorthalinRaidTools:LoadPlayersOnItem();
	else
		itemName:SetText("");
		iconIdEdit:SetText("");
		iconIdPic:SetImage(nil);
		playersPick:SetValue(nil);
		playersPick:SetList(nil);
	end
	playerName:SetText("");
	itemCount:SetText("");
end

function MorthalinRaidTools:LoadPlayersOnItem()
	local playerNames = {};
	for name, count in pairs(self.db.global.itemCounter[itemsPick.list[itemsPick:GetValue()]].players) do
		table.insert(playerNames, name);
	end
	local function sortNames(a, b)
		return a < b;
	end
	table.sort(playerNames, sortNames);
	playersPick:SetValue(nil);
	playersPick:SetList(nil);
	playerName:SetText("");
	itemCount:SetText("");
	for i = 1, #playerNames do
		playersPick:AddItem(i, "|cff" .. MorthalinRaidTools:GetClassColor(MorthalinRaidTools:GetPlayerInfo(playerNames[i]) == nil and "Unknown" or MorthalinRaidTools:GetPlayerInfo(playerNames[i]).class) .. playerNames[i]);
	end
	playersPick.playerNames = playerNames;
end

function MorthalinRaidTools:PlayerPicked()
	if(playersPick:GetValue() ~= nil and itemsPick:GetValue() ~= nil) then
		playerName:SetText(playersPick.playerNames[playersPick:GetValue()]);
		itemCount:SetText(self.db.global.itemCounter[itemsPick.list[itemsPick:GetValue()]].players[playersPick.playerNames[playersPick:GetValue()]]);
	else
		playerName:SetText("");
		itemCount:SetText("");
	end
end

function MorthalinRaidTools:AddPlayerToItem()
	self.db.global.itemCounter[itemsPick.list[itemsPick:GetValue()]].players["New player"] = 0;
	playersPick:AddItem(#playersPick.list + 1, "New player");
	table.insert(playersPick.playerNames, "New player");
	playerName:SetText("New player");
	itemCount:SetText("0");
	playersPick:SetValue(#playersPick.list);
end

function MorthalinRaidTools:RemovePlayerFromItem()
	self.db.global.itemCounter[itemsPick.list[itemsPick:GetValue()]].players[playersPick.playerNames[playersPick:GetValue()]] = nil;
	MorthalinRaidTools:LoadPlayersOnItem();
end

function MorthalinRaidTools:SetPlayerName(text)
	if(playersPick:GetValue() ~= nil and itemsPick:GetValue() ~= nil) then
		if(MorthalinRaidTools.db.global.itemCounter[itemsPick.list[itemsPick:GetValue()]].players[text] == nil) then
			playerName:SetText(text);
			MorthalinRaidTools:HideWhisperer();
			MorthalinRaidTools.db.global.itemCounter[itemsPick.list[itemsPick:GetValue()]].players[text] = self.db.global.itemCounter[itemsPick.list[itemsPick:GetValue()]].players[playersPick.playerNames[playersPick:GetValue()]];
			self.db.global.itemCounter[itemsPick.list[itemsPick:GetValue()]].players[playersPick.playerNames[playersPick:GetValue()]] = nil;
			playersPick.pullout.items[playersPick:GetValue()]:SetText("|cff" .. MorthalinRaidTools:GetClassColor(MorthalinRaidTools:GetPlayerInfo(text) == nil and "Unknown" or MorthalinRaidTools:GetPlayerInfo(text).class) .. text);
			playersPick.list[playersPick:GetValue()] = "|cff" .. MorthalinRaidTools:GetClassColor(MorthalinRaidTools:GetPlayerInfo(text) == nil and "Unknown" or MorthalinRaidTools:GetPlayerInfo(text).class) .. text;
			playersPick:SetText("|cff" .. MorthalinRaidTools:GetClassColor(MorthalinRaidTools:GetPlayerInfo(text) == nil and "Unknown" or MorthalinRaidTools:GetPlayerInfo(text).class) .. text);
			playersPick.playerNames[playersPick:GetValue()] = text;
		else
			message("Player '" .. text .. "' was already used!");
			playerName:SetText(playersPick.playerNames[playersPick:GetValue()]);
		end
	end
end

function MorthalinRaidTools:SetItemCount(text)
	local temp = text:gsub("%D+", "");
	if(#text == #temp) then
		self.db.global.itemCounter[itemsPick.list[itemsPick:GetValue()]].players[playersPick.playerNames[playersPick:GetValue()]] = #text == 0 and 0 or tonumber(text);
	else
		itemCount:SetText(self.db.global.itemCounter[itemsPick.list[itemsPick:GetValue()]].players[playersPick.playerNames[playersPick:GetValue()]]);
	end
end

function MorthalinRaidTools:ArchiveItemCouterData()
	if(#MorthalinRaidTools.db.global.archiveItemCounters == 0 or MorthalinRaidTools.db.global.archiveItemCounters[#MorthalinRaidTools.db.global.archiveItemCounters].archDate ~= date("%d.%m.%Y",GetServerTime())) then
		if(#MorthalinRaidTools.db.global.archiveItemCounters > MorthalinRaidTools.db.global.archiveItemCountersN) then
			table.remove(MorthalinRaidTools.db.global.archiveItemCounters, 1);
		end
		table.insert(MorthalinRaidTools.db.global.archiveItemCounters, {archive = MorthalinRaidTools.db.global.itemCounter, archDate = date("%d.%m.%Y",GetServerTime()), archDateInt = GetServerTime()});
		local function sortDates(a, b)
			return a.archDateInt < b.archDateInt;
		end
		table.sort(MorthalinRaidTools.db.global.archiveItemCounters, sortDates);
	end
end

function MorthalinRaidTools:RestoreItemData(selectedArchiveIndex)
	MorthalinRaidTools.db.global.itemCounter = MorthalinRaidTools.db.global.archiveItemCounters[selectedArchiveIndex].archive;
end
